﻿namespace NewGroup
{
    public class NewGroup : TabsStudioExt.ITabsStudioAddin
    {
        public NewGroup()
        {
            System.Windows.Controls.Image horizontalImage = new System.Windows.Controls.Image();
            horizontalImage.Source = new System.Windows.Media.Imaging.BitmapImage(
                new System.Uri("pack://application:,,,/NewGroup;component/horizontal.png"));

            System.Windows.Controls.Image verticalImage = new System.Windows.Controls.Image();
            verticalImage.Source = new System.Windows.Media.Imaging.BitmapImage(
                new System.Uri("pack://application:,,,/NewGroup;component/vertical.png"));

            newHorizontalTabGroupMenuItem = new System.Windows.Controls.MenuItem();
            newHorizontalTabGroupMenuItem.Icon = horizontalImage;
            newHorizontalTabGroupMenuItem.Header = "New horizontal tab group";
            newHorizontalTabGroupMenuItem.Click += NewHorizontalTabGroup;

            newVerticalTabGroupMenuItem = new System.Windows.Controls.MenuItem();
            newVerticalTabGroupMenuItem.Icon = verticalImage;
            newVerticalTabGroupMenuItem.Header = "New vertical tab group";
            newVerticalTabGroupMenuItem.Click += NewVerticalTabGroup;
        }

        public void OnConnection(TabsStudioExt.ITabsStudioEngine engine_, EnvDTE80.DTE2 dte_)
        {
            engine = engine_;
            dte = dte_;
            engine.OpeningTabExtensionContextMenu += OpeningTabExtensionContextMenu;
            engine.OpeningTabContextMenu += OpeningTabContextMenu;
        }

        public void OnDisconnection()
        {
            engine.OpeningTabExtensionContextMenu -= OpeningTabExtensionContextMenu;
            engine.OpeningTabContextMenu -= OpeningTabContextMenu;
        }

        private void OpeningTabExtensionContextMenu(object sender,
            TabsStudioExt.OpeningTabExtensionContextMenuEventArgs e)
        {
            if (e.TabExtension.IsExtensionActive)
                OpeningContextMenu(e.ContextMenu, e.Tab);
        }

        private void OpeningTabContextMenu(object sender, TabsStudioExt.OpeningTabContextMenuEventArgs e)
        {
            OpeningContextMenu(e.ContextMenu, e.Tab);
        }

        private void OpeningContextMenu(System.Windows.Controls.ContextMenu menu, TabsStudioExt.Tab tab)
        {
            if (tab.IsTabSelected &&
                (IsCommandAvailable(newHorizontalTabGroupCommand) || IsCommandAvailable(newVerticalTabGroupCommand)))
            {
                PlaceMenuItems(menu);
            }
        }
        
        private void PlaceMenuItems(System.Windows.Controls.ContextMenu menu)
        {
            foreach (object item in menu.Items)
            {
                System.Windows.Controls.Control i = item as System.Windows.Controls.Control;
                if (i == null)
                    continue;

                if (i.Name.Equals("TabsStudioSeparator") || i.Name.Equals("TabGroupsSeparator"))
                {
                    int index = menu.Items.IndexOf(item);
                    if (i.Name.Equals("TabsStudioSeparator"))
                        menu.Items.Insert(index, new System.Windows.Controls.Separator());
                    ++index;
                    if (IsCommandAvailable(newHorizontalTabGroupCommand))
                        menu.Items.Insert(index++, newHorizontalTabGroupMenuItem);
                    if (IsCommandAvailable(newVerticalTabGroupCommand))
                        menu.Items.Insert(index++, newVerticalTabGroupMenuItem);
                    break;
                }
            }
        }

        private void NewHorizontalTabGroup(object sender, System.Windows.RoutedEventArgs e)
        {
            ExecuteCommand("Window.NewHorizontalTabGroup");
        }

        private void NewVerticalTabGroup(object sender, System.Windows.RoutedEventArgs e)
        {
            ExecuteCommand("Window.NewVerticalTabGroup");
        }

        private void ExecuteCommand(string command)
        {
            dte.ExecuteCommand(command, "");
        }

        private bool IsCommandAvailable(string command)
        {
	        EnvDTE80.Commands2 commands = dte.Commands as EnvDTE80.Commands2;
	        if (commands == null)
		        return false;

            return commands.Item(command, 0).IsAvailable;
        }

        private TabsStudioExt.ITabsStudioEngine engine;
        private EnvDTE80.DTE2 dte;
        private System.Windows.Controls.MenuItem newHorizontalTabGroupMenuItem, newVerticalTabGroupMenuItem;
        private static string newHorizontalTabGroupCommand = "Window.NewHorizontalTabGroup";
        private static string newVerticalTabGroupCommand = "Window.NewVerticalTabGroup";
    }
}
